@@ -1,5 +1,6 @@
+from matpower import path_matpower
+
 from matpowercaseframes import CaseFrames
-from matpowercaseframes.idx import BUS_I, BUS_TYPE
 
 """
     pytest -n auto -rA --cov-report term --cov=matpowercaseframes tests/
@@ -10,10 +11,17 @@ def test_case9():
     CASE_NAME = 'case9.m'
     CaseFrames(CASE_NAME)
 
+
 def test_case4_dist():
     CASE_NAME = 'case4_dist.m'
     CaseFrames(CASE_NAME)
 
+
 def test_case118():
     CASE_NAME = 'case118.m'
     CaseFrames(CASE_NAME)
+
+
+def test_t_case9_dcline():
+    CASE_NAME = f"{path_matpower}/lib/t/t_case9_dcline.m"
+    CaseFrames(CASE_NAME)
